import React from 'react'
import { NavBar, Curation, Videos, Layout, Loading, Modals } from './components'
import { withRouter } from 'react-router-dom'
import socket from './socket'

const initialState = {
  loading: true,
  blocks: [],
  timestamp: 0,
  modal: null,
  validators: [],
  validator: null,
  videos: [],
  curations: [],
  showCurated: false,
  me: {
    page: 0,
    provider: '',
    channel: 0,
    video: 0,
    perPage: 12,
    hiddenChannels: {},
  },
  search: '',
}

class App extends React.Component {
  initializeSocket() {
    socket.on('connect', () => {
      if (!socket.id) return console.log('no websocket connection')
      console.debug('my socketId:', socket.id)
    })
    socket.on('config', ({ region, storageProviders }) => {
      const providers = storageProviders
      let { provider } = this.state.me
      if (provider === '') provider = providers[region][0]
      this.updateMe({ region, provider, providers })
    })
    socket.on('videos', (videos) => {
      console.debug(`received ${videos.length} videos`)
      this.setState({ loading: false })
      this.save({ videos })
    })
    socket.on('curations', (curations) => {
      if (!this.state.me.id) return
      console.debug(`received ${curations.length} curations!`)
      this.setState({ curations })
    })
    socket.on('curation', (curation: any) => {
      console.debug(`new curation`, curation)
      const curations = this.state.curations.concat(curation)
      this.setState({ curations })
      if (curation.videoId === this.state.video) this.playNext()
    })
    socket.on('blocks', (newblocks) => {
      const blocks = this.state.blocks
        .filter((b) => !newblocks.find((blk) => blk.id === b.id))
        .concat(newblocks)
        .sort((a, b) => b.id - a.id)
      this.setState({ loading: false, blocks }) // TODO
    })
    socket.on('block', (block) => {
      const blocks = this.state.blocks
        .filter((b) => b.id !== block.id)
        .concat(block)
        .sort((a, b) => b.id - a.id)
      this.setState({ loading: false, blocks, timestamp: block.timestamp })
    })
  }
  updateMe(object) {
    let { me } = this.state
    Object.keys(object).forEach((key) => (me[key] = object[key]))
    this.save({ me })
  }
  refreshVideos() {
    console.log(`clear`)
    socket.emit('get videos')
    this.updateMe({ hiddenChannels: {} })
  }
  handleChange(e: { taget: { name: string, value: string } }) {
    const { name, value } = e.target
    if (name === 'search') return this.setState({ [name]: value })
    if (name === 'perPage') this.selectPage(0)
    this.updateMe({ [name]: value })
  }
  toggleShowCurated() {
    this.setState({ showCurated: !this.state.showCurated })
  }
  setShowModal(showModal, vars = {}) {
    this.updateMe({ showModal })
    this.setState({ ...vars })
  }
  showValidator(account) {
    const { validators } = this.state
    const v = { account }
    const validator = validators.find((v) => v.account === account) || v
    this.setShowModal('validator', { validator })
  }
  toggleChannel(id: number) {
    let { hiddenChannels } = this.state.me
    hiddenChannels[id] = !hiddenChannels[id]
    this.updateMe({ hiddenChannels })
  }
  selectPage(page: number) {
    this.updateMe({ page })
  }
  selectChannel(channel: number) {
    this.updateMe({ channel, page: 0 })
  }
  selectVideo(video: number) {
    this.updateMe({ showModal: 'player', video })
  }
  selectProvider(provider: string) {
    this.updateMe({ provider })
  }
  addVideoVote(videoId: number, vote: string) {
    if (this.state.curations.find((c) => c.videoId === videoId)) {
      console.log(`curation for video ${videoId} exists`)
      // TODO `I changed my mind` => update existing vote, add rationale
      return this.playNext()
    } else {
      const memberId = 515 // TODO
      socket.emit('addVideoVote', { videoId, vote, memberId })
    }
  }
  playNext() {
    const { curations, videos } = this.state
    const next = this.filterVideos(videos, curations)[0]
    if (next) this.setState({ video: next.id })
    else this.setShowModal()
  }
  filterHiddenChannels(videos, hiddenChannels) {
    if (!hiddenChannels) return videos
    const hidden = Object.keys(hiddenChannels)
    return videos.filter((v) => !hidden.find((h) => Number(h) === v.channelId))
  }
  filterVideos() {
    const { videos, curations, search, showCurated, me } = this.state
    let filtered = this.filterHiddenChannels(videos, me.hiddenChannels)
    if (me.channel)
      filtered = filtered.filter((v) => v.channelId === me.channel)
    if (search !== '')
      filtered = filtered.filter(({ title, description }) =>
        `${title} ${description}`.toLowerCase().includes(search.toLowerCase())
      )

    if (!me.id) return filtered // curator

    const unreviewed = filtered.filter(
      (video) => showCurated || !curations.find((c) => c.videoId === video.id)
    )
    if (search === '') return unreviewed

    const findCurations = (id: number) =>
      curations
        .filter((c) => c.videoId === id)
        .map((c) => c.vote)
        .join()

    return unreviewed.filter(({ id, title, description }) =>
      `${title} ${description} ${findCurations(id)}`.includes(search)
    )
  }

  renderError() {
    if (this.state.showModal === 'Error') return
    this.setShowModal('Error')
  }
  renderApp() {
    const {
      showModal,
      curations,
      video,
      videos,
      validator,
      me,
      search,
    } = this.state
    return (
      <div className="w-100 h-100 d-flex flex-column bg-dark overflow-hidden">
        <NavBar
          handleChange={this.handleChange}
          refreshVideos={this.refreshVideos}
          setPerPage={this.selectPerPage}
          videos={videos.length}
          search={search}
          me={me}
        />
        <Modals
          handleChange={this.handleChange}
          setShowModal={this.setShowModal}
          showModal={showModal}
          validator={validator}
          selectProvider={this.selectProvider}
          addVideoVote={this.addVideoVote}
          blocks={this.state.blocks}
          videos={videos}
          me={me}
        />
        {me.id ? (
          <Curation
            toggleShowCurated={this.toggleShowCurated}
            handleChange={this.handleChange}
            filterVideos={this.filterVideos}
            selectVideo={this.selectVideo}
            addVideoVote={this.addVideoVote}
            videos={this.filterVideos(videos)}
            video={video}
            curations={curations}
            showCurated={this.state.showCurated}
            search={search}
          />
        ) : (
          <Videos
            handleChange={this.handleChange}
            hideChannel={this.toggleChannel}
            selectPage={this.selectPage}
            selectChannel={this.selectChannel}
            selectVideo={this.selectVideo}
            videos={this.filterVideos(videos)}
            search={search}
            me={me}
          />
        )}
      </div>
    )
  }
  render() {
    if (this.state.loading) return <Loading />
    if (this.state.error) this.renderError()
    if (this.state.component === 'layout') return <Layout {...this.state} />
    return this.renderApp()
  }

  load(key: string) {
    console.debug(`loading ${key}`)
    try {
      const data = localStorage.getItem(key)
      if (data) this.setState({ [key]: JSON.parse(data) })
    } catch (e) {
      console.warn(`Failed to load ${key}`, e)
    }
  }
  save(object) {
    Object.keys(object).forEach((key) => {
      const value = object[key]
      if (!value) return
      try {
        localStorage.setItem(key, JSON.stringify(value))
      } catch (e) {
        console.warn(`Failed to save ${key}`, e)
      } finally {
        this.setState({ [key]: value })
      }
    })
  }

  loadData() {
    this.load('me')
    this.load('videos')
  }

  componentDidMount() {
    this.initializeSocket()
    this.loadData()
  }
  componentWillUnmount() {
    console.log('unmounting...')
  }
  constructor() {
    super()
    this.state = initialState
    this.refreshVideos = this.refreshVideos.bind(this)
    this.setShowModal = this.setShowModal.bind(this)
    this.showValidator = this.showValidator.bind(this)
    this.addVideoVote = this.addVideoVote.bind(this)
    this.toggleChannel = this.toggleChannel.bind(this)
    this.selectPage = this.selectPage.bind(this)
    this.selectChannel = this.selectChannel.bind(this)
    this.selectVideo = this.selectVideo.bind(this)
    this.selectProvider = this.selectProvider.bind(this)
    this.filterVideos = this.filterVideos.bind(this)
    this.handleChange = this.handleChange.bind(this)
    this.toggleShowCurated = this.toggleShowCurated.bind(this)
  }
}

export default withRouter(App)
